# -*- coding: utf-8 -*-
#
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Unit tests."""

import mock
import pytest

from google.android import overtheair_v1
from google.android.overtheair_v1.proto import overtheair_data_pb2
from google.android.overtheair_v1.proto import overtheair_service_messages_pb2


class MultiCallableStub(object):
    """Stub for the grpc.UnaryUnaryMultiCallable interface."""

    def __init__(self, method, channel_stub):
        self.method = method
        self.channel_stub = channel_stub

    def __call__(self, request, timeout=None, metadata=None, credentials=None):
        self.channel_stub.requests.append((self.method, request))

        response = None
        if self.channel_stub.responses:
            response = self.channel_stub.responses.pop()

        if isinstance(response, Exception):
            raise response

        if response:
            return response


class ChannelStub(object):
    """Stub for the grpc.Channel interface."""

    def __init__(self, responses=[]):
        self.responses = responses
        self.requests = []

    def unary_unary(self,
                    method,
                    request_serializer=None,
                    response_deserializer=None):
        return MultiCallableStub(method, self)


class CustomException(Exception):
    pass


class TestOverTheAirServiceClient(object):
    def test_get_deployment(self):
        # Setup Expected Response
        name_2 = 'name2-1052831874'
        description = 'description-1724546052'
        api_level = 81920737
        apfe_product_id = 'apfeProductId-1747882468'
        expected_response = {
            'name': name_2,
            'description': description,
            'api_level': api_level,
            'apfe_product_id': apfe_product_id
        }
        expected_response = overtheair_data_pb2.Deployment(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.deployment_path('[DEPLOYMENT]')

        response = client.get_deployment(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.GetDeploymentRequest(
            name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_get_deployment_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.deployment_path('[DEPLOYMENT]')

        with pytest.raises(CustomException):
            client.get_deployment(name)

    def test_list_deployments(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = overtheair_service_messages_pb2.ListDeploymentsResponse(
            **expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        response = client.list_deployments()
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.ListDeploymentsRequest(
        )
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_list_deployments_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        with pytest.raises(CustomException):
            client.list_deployments()

    def test_update_deployment(self):
        # Setup Expected Response
        name_2 = 'name2-1052831874'
        description = 'description-1724546052'
        api_level = 81920737
        apfe_product_id = 'apfeProductId-1747882468'
        expected_response = {
            'name': name_2,
            'description': description,
            'api_level': api_level,
            'apfe_product_id': apfe_product_id
        }
        expected_response = overtheair_data_pb2.Deployment(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.deployment_path('[DEPLOYMENT]')
        deployment = {}
        version_tag = {}

        response = client.update_deployment(name, deployment, version_tag)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.UpdateDeploymentRequest(
            name=name, deployment=deployment, version_tag=version_tag)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_update_deployment_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.deployment_path('[DEPLOYMENT]')
        deployment = {}
        version_tag = {}

        with pytest.raises(CustomException):
            client.update_deployment(name, deployment, version_tag)

    def test_get_package(self):
        # Setup Expected Response
        name_2 = 'name2-1052831874'
        title = 'title110371416'
        expected_response = {'name': name_2, 'title': title}
        expected_response = overtheair_data_pb2.Package(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.package_path_path('[DEPLOYMENT]', '[PACKAGE_PATH]')

        response = client.get_package(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.GetPackageRequest(
            name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_get_package_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.package_path_path('[DEPLOYMENT]', '[PACKAGE_PATH]')

        with pytest.raises(CustomException):
            client.get_package(name)

    def test_list_packages(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = overtheair_service_messages_pb2.ListPackagesResponse(
            **expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.deployment_path('[DEPLOYMENT]')

        response = client.list_packages(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.ListPackagesRequest(
            name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_list_packages_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.deployment_path('[DEPLOYMENT]')

        with pytest.raises(CustomException):
            client.list_packages(name)

    def test_delete_package(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = overtheair_service_messages_pb2.DeletePackageResponse(
            **expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.package_path_path('[DEPLOYMENT]', '[PACKAGE_PATH]')

        response = client.delete_package(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.DeletePackageRequest(
            name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_delete_package_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.package_path_path('[DEPLOYMENT]', '[PACKAGE_PATH]')

        with pytest.raises(CustomException):
            client.delete_package(name)

    def test_update_package(self):
        # Setup Expected Response
        name_2 = 'name2-1052831874'
        title = 'title110371416'
        expected_response = {'name': name_2, 'title': title}
        expected_response = overtheair_data_pb2.Package(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.package_path_path('[DEPLOYMENT]', '[PACKAGE_PATH]')
        package_obj = {}
        version_tag = {}

        response = client.update_package(name, package_obj, version_tag)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.UpdatePackageRequest(
            name=name, package_obj=package_obj, version_tag=version_tag)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_update_package_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.package_path_path('[DEPLOYMENT]', '[PACKAGE_PATH]')
        package_obj = {}
        version_tag = {}

        with pytest.raises(CustomException):
            client.update_package(name, package_obj, version_tag)

    def test_create_config(self):
        # Setup Expected Response
        name_2 = 'name2-1052831874'
        title = 'title110371416'
        description = 'description-1724546052'
        update_battery_awareness = False
        update_required_setup = False
        immediate_release = False
        lower_checkin_interval = False
        security_update = True
        expected_response = {
            'name': name_2,
            'title': title,
            'description': description,
            'update_battery_awareness': update_battery_awareness,
            'update_required_setup': update_required_setup,
            'immediate_release': immediate_release,
            'lower_checkin_interval': lower_checkin_interval,
            'security_update': security_update
        }
        expected_response = overtheair_data_pb2.Config(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.deployment_path('[DEPLOYMENT]')
        config = {}

        response = client.create_config(name, config)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.CreateConfigRequest(
            name=name, config=config)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_create_config_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.deployment_path('[DEPLOYMENT]')
        config = {}

        with pytest.raises(CustomException):
            client.create_config(name, config)

    def test_get_config(self):
        # Setup Expected Response
        name_2 = 'name2-1052831874'
        title = 'title110371416'
        description = 'description-1724546052'
        update_battery_awareness = False
        update_required_setup = False
        immediate_release = False
        lower_checkin_interval = False
        security_update = True
        expected_response = {
            'name': name_2,
            'title': title,
            'description': description,
            'update_battery_awareness': update_battery_awareness,
            'update_required_setup': update_required_setup,
            'immediate_release': immediate_release,
            'lower_checkin_interval': lower_checkin_interval,
            'security_update': security_update
        }
        expected_response = overtheair_data_pb2.Config(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.config_path('[DEPLOYMENT]', '[CONFIG]')

        response = client.get_config(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.GetConfigRequest(
            name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_get_config_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.config_path('[DEPLOYMENT]', '[CONFIG]')

        with pytest.raises(CustomException):
            client.get_config(name)

    def test_list_configs(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = overtheair_service_messages_pb2.ListConfigsResponse(
            **expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.deployment_path('[DEPLOYMENT]')

        response = client.list_configs(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.ListConfigsRequest(
            name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_list_configs_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.deployment_path('[DEPLOYMENT]')

        with pytest.raises(CustomException):
            client.list_configs(name)

    def test_delete_config(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = overtheair_service_messages_pb2.DeleteConfigResponse(
            **expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.config_path('[DEPLOYMENT]', '[CONFIG]')

        response = client.delete_config(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.DeleteConfigRequest(
            name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_delete_config_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.config_path('[DEPLOYMENT]', '[CONFIG]')

        with pytest.raises(CustomException):
            client.delete_config(name)

    def test_update_config(self):
        # Setup Expected Response
        name_2 = 'name2-1052831874'
        title = 'title110371416'
        description = 'description-1724546052'
        update_battery_awareness = False
        update_required_setup = False
        immediate_release = False
        lower_checkin_interval = False
        security_update = True
        expected_response = {
            'name': name_2,
            'title': title,
            'description': description,
            'update_battery_awareness': update_battery_awareness,
            'update_required_setup': update_required_setup,
            'immediate_release': immediate_release,
            'lower_checkin_interval': lower_checkin_interval,
            'security_update': security_update
        }
        expected_response = overtheair_data_pb2.Config(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.config_path('[DEPLOYMENT]', '[CONFIG]')
        config = {}
        version_tag = {}

        response = client.update_config(name, config, version_tag)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.UpdateConfigRequest(
            name=name, config=config, version_tag=version_tag)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_update_config_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.config_path('[DEPLOYMENT]', '[CONFIG]')
        config = {}
        version_tag = {}

        with pytest.raises(CustomException):
            client.update_config(name, config, version_tag)

    def test_create_group(self):
        # Setup Expected Response
        name_2 = 'name2-1052831874'
        enabled = False
        bypass_rollout_schedule = True
        expected_response = {
            'name': name_2,
            'enabled': enabled,
            'bypass_rollout_schedule': bypass_rollout_schedule
        }
        expected_response = overtheair_data_pb2.Group(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.deployment_path('[DEPLOYMENT]')
        group = {}

        response = client.create_group(name, group)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.CreateGroupRequest(
            name=name, group=group)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_create_group_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.deployment_path('[DEPLOYMENT]')
        group = {}

        with pytest.raises(CustomException):
            client.create_group(name, group)

    def test_get_group(self):
        # Setup Expected Response
        name_2 = 'name2-1052831874'
        enabled = False
        bypass_rollout_schedule = True
        expected_response = {
            'name': name_2,
            'enabled': enabled,
            'bypass_rollout_schedule': bypass_rollout_schedule
        }
        expected_response = overtheair_data_pb2.Group(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.group_path('[DEPLOYMENT]', '[GROUP]')

        response = client.get_group(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.GetGroupRequest(
            name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_get_group_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.group_path('[DEPLOYMENT]', '[GROUP]')

        with pytest.raises(CustomException):
            client.get_group(name)

    def test_list_groups(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = overtheair_service_messages_pb2.ListGroupsResponse(
            **expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.deployment_path('[DEPLOYMENT]')

        response = client.list_groups(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.ListGroupsRequest(
            name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_list_groups_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.deployment_path('[DEPLOYMENT]')

        with pytest.raises(CustomException):
            client.list_groups(name)

    def test_delete_group(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = overtheair_service_messages_pb2.DeleteGroupResponse(
            **expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.group_path('[DEPLOYMENT]', '[GROUP]')

        response = client.delete_group(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.DeleteGroupRequest(
            name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_delete_group_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.group_path('[DEPLOYMENT]', '[GROUP]')

        with pytest.raises(CustomException):
            client.delete_group(name)

    def test_update_group(self):
        # Setup Expected Response
        name_2 = 'name2-1052831874'
        enabled = False
        bypass_rollout_schedule = True
        expected_response = {
            'name': name_2,
            'enabled': enabled,
            'bypass_rollout_schedule': bypass_rollout_schedule
        }
        expected_response = overtheair_data_pb2.Group(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.group_path('[DEPLOYMENT]', '[GROUP]')
        group = {}
        version_tag = {}

        response = client.update_group(name, group, version_tag)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.UpdateGroupRequest(
            name=name, group=group, version_tag=version_tag)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_update_group_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.group_path('[DEPLOYMENT]', '[GROUP]')
        group = {}
        version_tag = {}

        with pytest.raises(CustomException):
            client.update_group(name, group, version_tag)

    def test_disable_groups(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = overtheair_service_messages_pb2.DisableGroupsResponse(
            **expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.deployment_path('[DEPLOYMENT]')
        group_names = []

        response = client.disable_groups(name, group_names)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.DisableGroupsRequest(
            name=name, group_names=group_names)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_disable_groups_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.deployment_path('[DEPLOYMENT]')
        group_names = []

        with pytest.raises(CustomException):
            client.disable_groups(name, group_names)

    def test_enable_groups(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = overtheair_service_messages_pb2.EnableGroupsResponse(
            **expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.deployment_path('[DEPLOYMENT]')
        group_names = []

        response = client.enable_groups(name, group_names)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.EnableGroupsRequest(
            name=name, group_names=group_names)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_enable_groups_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.deployment_path('[DEPLOYMENT]')
        group_names = []

        with pytest.raises(CustomException):
            client.enable_groups(name, group_names)

    def test_create_group_member(self):
        # Setup Expected Response
        name_2 = 'name2-1052831874'
        expected_response = {'name': name_2}
        expected_response = overtheair_data_pb2.GroupMember(
            **expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.group_path('[DEPLOYMENT]', '[GROUP]')
        group_member = {}

        response = client.create_group_member(name, group_member)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.CreateGroupMemberRequest(
            name=name, group_member=group_member)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_create_group_member_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.group_path('[DEPLOYMENT]', '[GROUP]')
        group_member = {}

        with pytest.raises(CustomException):
            client.create_group_member(name, group_member)

    def test_create_group_members(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = overtheair_service_messages_pb2.CreateGroupMembersResponse(
            **expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.group_path('[DEPLOYMENT]', '[GROUP]')
        group_members = []

        response = client.create_group_members(name, group_members)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.CreateGroupMembersRequest(
            name=name, group_members=group_members)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_create_group_members_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.group_path('[DEPLOYMENT]', '[GROUP]')
        group_members = []

        with pytest.raises(CustomException):
            client.create_group_members(name, group_members)

    def test_get_group_member(self):
        # Setup Expected Response
        name_2 = 'name2-1052831874'
        expected_response = {'name': name_2}
        expected_response = overtheair_data_pb2.GroupMember(
            **expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.member_path('[DEPLOYMENT]', '[GROUP]', '[MEMBER]')

        response = client.get_group_member(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.GetGroupMemberRequest(
            name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_get_group_member_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.member_path('[DEPLOYMENT]', '[GROUP]', '[MEMBER]')

        with pytest.raises(CustomException):
            client.get_group_member(name)

    def test_list_group_members(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = overtheair_service_messages_pb2.ListGroupMembersResponse(
            **expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.group_path('[DEPLOYMENT]', '[GROUP]')

        response = client.list_group_members(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.ListGroupMembersRequest(
            name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_list_group_members_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.group_path('[DEPLOYMENT]', '[GROUP]')

        with pytest.raises(CustomException):
            client.list_group_members(name)

    def test_delete_group_member(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = overtheair_service_messages_pb2.DeleteGroupMemberResponse(
            **expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.member_path('[DEPLOYMENT]', '[GROUP]', '[MEMBER]')

        response = client.delete_group_member(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.DeleteGroupMemberRequest(
            name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_delete_group_member_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.member_path('[DEPLOYMENT]', '[GROUP]', '[MEMBER]')

        with pytest.raises(CustomException):
            client.delete_group_member(name)

    def test_delete_group_members(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = overtheair_service_messages_pb2.DeleteGroupMembersResponse(
            **expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.group_path('[DEPLOYMENT]', '[GROUP]')
        group_members = []

        response = client.delete_group_members(name, group_members)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.DeleteGroupMembersRequest(
            name=name, group_members=group_members)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_delete_group_members_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.group_path('[DEPLOYMENT]', '[GROUP]')
        group_members = []

        with pytest.raises(CustomException):
            client.delete_group_members(name, group_members)

    def test_get_group_chart(self):
        # Setup Expected Response
        name_2 = 'name2-1052831874'
        expected_response = {'name': name_2}
        expected_response = overtheair_data_pb2.Chart(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.chart_path('[DEPLOYMENT]', '[GROUP]', '[CHART]')

        response = client.get_group_chart(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.GetGroupChartRequest(
            name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_get_group_chart_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.chart_path('[DEPLOYMENT]', '[GROUP]', '[CHART]')

        with pytest.raises(CustomException):
            client.get_group_chart(name)

    def test_list_group_charts(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = overtheair_service_messages_pb2.ListGroupChartsResponse(
            **expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.group_path('[DEPLOYMENT]', '[GROUP]')

        response = client.list_group_charts(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.ListGroupChartsRequest(
            name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_list_group_charts_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.group_path('[DEPLOYMENT]', '[GROUP]')

        with pytest.raises(CustomException):
            client.list_group_charts(name)

    def test_list_group_configs(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = overtheair_service_messages_pb2.ListGroupConfigsResponse(
            **expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.group_path('[DEPLOYMENT]', '[GROUP]')

        response = client.list_group_configs(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.ListGroupConfigsRequest(
            name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_list_group_configs_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.group_path('[DEPLOYMENT]', '[GROUP]')

        with pytest.raises(CustomException):
            client.list_group_configs(name)

    def test_update_group_configs(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = overtheair_service_messages_pb2.UpdateGroupConfigsResponse(
            **expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.group_path('[DEPLOYMENT]', '[GROUP]')
        config_names = []
        version_tag = {}

        response = client.update_group_configs(name, config_names, version_tag)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.UpdateGroupConfigsRequest(
            name=name, config_names=config_names, version_tag=version_tag)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_update_group_configs_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.group_path('[DEPLOYMENT]', '[GROUP]')
        config_names = []
        version_tag = {}

        with pytest.raises(CustomException):
            client.update_group_configs(name, config_names, version_tag)

    def test_disable_all_enabled_groups_of_config(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = overtheair_service_messages_pb2.DisableAllEnabledGroupsOfConfigResponse(
            **expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.config_path('[DEPLOYMENT]', '[CONFIG]')

        response = client.disable_all_enabled_groups_of_config(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.DisableAllEnabledGroupsOfConfigRequest(
            name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_disable_all_enabled_groups_of_config_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.config_path('[DEPLOYMENT]', '[CONFIG]')

        with pytest.raises(CustomException):
            client.disable_all_enabled_groups_of_config(name)

    def test_list_config_packages(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = overtheair_service_messages_pb2.ListConfigPackagesResponse(
            **expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.config_path('[DEPLOYMENT]', '[CONFIG]')

        response = client.list_config_packages(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.ListConfigPackagesRequest(
            name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_list_config_packages_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.config_path('[DEPLOYMENT]', '[CONFIG]')

        with pytest.raises(CustomException):
            client.list_config_packages(name)

    def test_update_config_packages(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = overtheair_service_messages_pb2.UpdateConfigPackagesResponse(
            **expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.config_path('[DEPLOYMENT]', '[CONFIG]')
        package_configs = []

        response = client.update_config_packages(name, package_configs)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.UpdateConfigPackagesRequest(
            name=name, package_configs=package_configs)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_update_config_packages_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.config_path('[DEPLOYMENT]', '[CONFIG]')
        package_configs = []

        with pytest.raises(CustomException):
            client.update_config_packages(name, package_configs)

    def test_list_configs_using_package(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = overtheair_service_messages_pb2.ListConfigsUsingPackageResponse(
            **expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup Request
        name = client.package_path_path('[DEPLOYMENT]', '[PACKAGE_PATH]')

        response = client.list_configs_using_package(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = overtheair_service_messages_pb2.ListConfigsUsingPackageRequest(
            name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_list_configs_using_package_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch('google.api_core.grpc_helpers.create_channel')
        with patch as create_channel:
            create_channel.return_value = channel
            client = overtheair_v1.OverTheAirServiceClient()

        # Setup request
        name = client.package_path_path('[DEPLOYMENT]', '[PACKAGE_PATH]')

        with pytest.raises(CustomException):
            client.list_configs_using_package(name)
